@@ -1,15 +1,4 @@
 # function to run a topic model with specified parameters
-# old name
-run_LDA <- function(
-  dtm,
-  type = "lda",
-  n_topics = 5,
-  iterations = 2000
-){
-  run_topic_model(dtm, type, n_topics, iterations)
-}
-
-# new name
 run_topic_model <- function(
 	dtm,
 	type = "lda",
@@ -17,7 +6,8 @@ run_topic_model <- function(
 	iterations = 2000
 ){
 	LDA_control <- list(
-    iter = iterations
+    iter = iterations,
+    burnin = iterations * 0.1
   )
 	switch(type,
 		"ctm" = {
