@@ -80,25 +80,21 @@ build_plot_data <- function(info, dtm, model, hide_names){
   x_matrix <- modeltools::posterior(model)$topics # article x topic
   y_matrix <- t(modeltools::posterior(model)$terms)
 
-  # exclude following columns: topic, select, display
-  keep_cols <- which((colnames(info) %in% c("topic", "selected", "display")) == FALSE)
-
   # build main plot information (x)
   x_df <- cbind(
-    info[, keep_cols],
+    info,
     data.frame(
-      topic = apply(x_matrix, 1, which.max),
-      ade4::dudi.coa(x_matrix, scannf = FALSE, nf=3)$li
+      ade4::dudi.coa(x_matrix, scannf = FALSE, nf = 3)$li
     )
   )
+  x_df$topic <- apply(x_matrix, 1, which.max)
 
   # add citation in correctly formatted way
   x_df$caption <- paste0(
-    add_line_breaks(
-      format_citation(
-        data = x_df,
-        details = (hide_names == FALSE)
-      )
+    format_citation(
+      data = x_df,
+      details = (hide_names == FALSE),
+      line_breaks = TRUE
     ),
     "<br>[Topic #",
     x_df$topic,
@@ -111,6 +107,12 @@ build_plot_data <- function(info, dtm, model, hide_names){
     )
   })
 
+  # restrict to useful columns
+  x_df <- x_df[, colnames(x_df) %in% c(
+    "label", "author", "year", "title", "journal", "pages", "abstract",
+    "Axis1", "Axis2", "Axis3", "topic", "caption", "common_words")
+  ]
+
   # build word plot information (y)
   y_df <- data.frame(
     term = rep(
@@ -200,7 +202,7 @@ build_plot_data <- function(info, dtm, model, hide_names){
 
 add_required_columns <- function(data){
   added_cols <- data.frame(
-    selected = NA,
+    screened_topics = NA,
     display = TRUE,
     topic = NA,
     notes = NA
@@ -223,8 +225,10 @@ add_required_columns <- function(data){
 get_topic_colnames <- function(data){
   colnames(data)[
     which(
-      (colnames(data) %in%
-      c("selected", "topic", "display", "notes")) == FALSE
+      !(colnames(data) %in%
+      c("screened_topics", "screened_titles", "screened_abstracts",
+        "topic", "display", "notes"
+      ))
     )
   ]
 }
@@ -248,7 +252,7 @@ build_appearance <- function(plot_data, palette){
 update_appearance <- function(plot_data, palette){
   lapply(plot_data, function(a, colours){
     rows <- which(
-      (a$color %in% c("#000000", "#CCCCCC")) == FALSE
+      !(a$color %in% c("#000000", "#CCCCCC"))
     )
     a$color[rows] <- colours[a$topic[rows]]
     return(a)
